import { writable } from 'svelte/store';
import { type ActionReturnType, actions } from 'astro:actions';
import { actionErrorPopup } from '@util/action.ts';
import { addToWritableArray, deleteFromWritableArray, updateWritableArray } from '@util/state.ts';

// types
export type Users = Exclude<ActionReturnType<typeof actions.user.users>['data'], undefined>['users'];
export type User = Users[0];

// state
export const users = writable<Users>([]);

// actions
export async function fetchUsers(options?: { username?: string | null }) {
  const { data, error } = await actions.user.users({ username: options?.username });
  if (error) {
    actionErrorPopup(error);
    return;
  }

  users.set(data.users);
}

export async function addUser(user: User) {
  const { data, error } = await actions.user.addUser({
    username: user.username,
    firstname: user.firstname,
    lastname: user.lastname,
    birthday: user.birthday,
    telephone: user.telephone,
    uuid: user.uuid
  });
  if (error) {
    actionErrorPopup(error);
    return;
  }

  addToWritableArray(users, user);
}

export async function editUser(user: User) {
  const { error } = await actions.user.editUser({
    id: user.id,
    username: user.username,
    firstname: user.firstname,
    lastname: user.lastname,
    birthday: user.birthday,
    telephone: user.telephone,
    uuid: user.uuid
  });
  if (error) {
    actionErrorPopup(error);
    return;
  }

  updateWritableArray(users, user, (t) => t.id == user.id);
}

export async function deleteUser(user: User) {
  const { error } = await actions.user.deleteUser({ id: user.id });
  if (error) {
    actionErrorPopup(error);
    return;
  }

  deleteFromWritableArray(users, (t) => t.id == user.id);
}
